@@ -5,7 +5,7 @@ nepali_datetime.
 
 Supports >= Python3.5
 """
-__version__ = "1.0.5.1"
+__version__ = "1.0.7"
 
 __author__ = "Amit Garu <amitgaru2@gmail.com>"
 
@@ -15,37 +15,49 @@ import time as _time
 import math as _math
 import datetime as _actual_datetime
 
-from nepali_datetime.config import CALENDAR_PATH, MINDATE, MAXDATE, REFERENCE_DATE_AD
+from .config import CALENDAR_PATH, MINDATE, MAXDATE, REFERENCE_DATE_AD
 
 MINYEAR = MINDATE['year']
 MAXYEAR = MAXDATE['year']
 
 NEPAL_TIME_UTC_OFFSET = 20700
 
-_MONTHNAMES = [None, "Bai", "Jes", "Asa", "Shr", "Bha", "Asw", "Kar", "Man", "Pou", "Mag", "Fal", "Cha"]
-_FULLMONTHNAMES = [None, "Baishakh", "Jestha", "Asar", "Shrawan", "Bhadau", "Aswin", "Kartik", "Mangsir", "Poush",
-                   "Magh", "Falgun", "Chaitra"]
-_MONTHNAMES_NP = [None, "वैशाख", "जेष्ठ", "असार", "श्रावण", "भदौ", "आश्विन", "कार्तिक", "मंसिर", "पौष",
-                  "माघ", "फाल्गुण", "चैत्र"]
-_DAYNAMES = [None, "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
-_FULLDAYNAMES = [None, "Monday", "Tueday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]
+_MONTHNAMES = (None, "Bai", "Jes", "Asa", "Shr", "Bha", "Asw", "Kar", "Man", "Pou", "Mag", "Fal", "Cha")
+_FULLMONTHNAMES = (None, "Baishakh", "Jestha", "Asar", "Shrawan", "Bhadau", "Aswin", "Kartik", "Mangsir", "Poush",
+                   "Magh", "Falgun", "Chaitra")
+_MONTHNAMES_NP = (None, "वैशाख", "जेष्ठ", "असार", "श्रावण", "भदौ", "आश्विन", "कार्तिक", "मंसिर", "पौष",
+                  "माघ", "फाल्गुण", "चैत्र")
+_DAYNAMES = (None, "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun")
+_FULLDAYNAMES = (None, "Monday", "Tueday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")
+_FULLDAYNAMES_NP = (None, "सोमबार", "मंगलबार", "बुधवार", "बिहिबार", "शुक्रबार", "शनिबार", "आइतबार")
+_DIGIT_NP = "०१२३४५६७८९"
+_EPOCH = _actual_datetime.datetime(1970, 1, 1, tzinfo=_actual_datetime.timezone.utc)
 
 _STRFTIME_CUSTOM_MAP = {
     'a': lambda o: '%s' % _DAYNAMES[(o.weekday() % 7) or 7],
     'A': lambda o: '%s' % _FULLDAYNAMES[(o.weekday() % 7) or 7],
+    'G': lambda o: '%s' % _FULLDAYNAMES_NP[(o.weekday() % 7) or 7],
     'w': lambda o: '%d' % o.weekday(),
     'd': lambda o: '%02d' % o.day,
+    'D': lambda o: ''.join(_DIGIT_NP[int(i)] for i in '%02d' % o.day),
     'b': lambda o: '%s' % _MONTHNAMES[o.month],
     'B': lambda o: '%s' % _FULLMONTHNAMES[o.month],
     'N': lambda o: '%s' % _MONTHNAMES_NP[o.month],
     'm': lambda o: '%02d' % o.month,
+    'n': lambda o: ''.join(_DIGIT_NP[int(i)] for i in '%02d' % o.month),
     'y': lambda o: '%02d' % (o.year % 100),
     'Y': lambda o: '%d' % o.year,
+    'k': lambda o: ''.join(_DIGIT_NP[int(i)] for i in '%02d' % (o.year % 100)),
+    'K': lambda o: ''.join(_DIGIT_NP[int(i)] for i in '%d' % o.year),
     'H': lambda o: '%02d' % getattr(o, 'hour', 0),
+    'h': lambda o: ''.join(_DIGIT_NP[int(i)] for i in '%02d' % getattr(o, 'hour', 0)),
     'I': lambda o: '%02d' % (getattr(o, 'hour', 0) % 12,),
+    'i': lambda o: ''.join(_DIGIT_NP[int(i)] for i in '%02d' % (getattr(o, 'hour', 0) % 12,)),
     'p': lambda o: 'AM' if getattr(o, 'hour', 0) < 12 else 'PM',
     'M': lambda o: '%02d' % getattr(o, 'minute', 0),
+    'l': lambda o: ''.join(_DIGIT_NP[int(i)] for i in '%02d' % getattr(o, 'minute', 0)),
     'S': lambda o: '%02d' % getattr(o, 'second', 0),
+    's': lambda o: ''.join(_DIGIT_NP[int(i)] for i in '%02d' % getattr(o, 'second', 0)),
 }
 
 _CALENDAR = {}
@@ -75,7 +87,7 @@ def _format_time(hh, mm, ss, us):
     return result
 
 
-def _wrap_strftime(object, format, timetuple):
+def _wrap_strftime(object, format):
     # Don't call utcoffset() or tzname() unless actually needed.
     freplace = None  # the string to use for %f
     zreplace = None  # the string to use for %z
@@ -121,7 +133,7 @@ def _wrap_strftime(object, format, timetuple):
                                 # strftime is going to have at this: escape %
                                 Zreplace = s.replace('%', '%%')
                     newformat.append(Zreplace)
-                elif ch in ('a', 'A', 'w', 'd', 'b', 'B', 'N', 'm', 'y', 'Y', 'H', 'I', 'p', 'M', 'S'):
+                elif ch in _STRFTIME_CUSTOM_MAP.keys():
                     newformat.append(_STRFTIME_CUSTOM_MAP[ch](object))
                 else:
                     push('%')
@@ -290,8 +302,8 @@ class UTC0545(_actual_datetime.tzinfo):
     def fromutc(self, dt):
         """datetime in UTC -> datetime in local time."""
 
-        if not isinstance(dt, datetime):
-            raise TypeError("fromutc() requires a nepali_datetime.datetime argument")
+        if not isinstance(dt, datetime) and not isinstance(dt, _actual_datetime.datetime):
+            raise TypeError("fromutc() requires a nepali_datetime.datetime or datetime.datetime argument")
         if dt.tzinfo is not self:
             raise ValueError("dt.tzinfo is not self")
 
@@ -426,7 +438,7 @@ class date:
 
     def strftime(self, fmt):
         """Format using strftime()."""
-        return _wrap_strftime(self, fmt, self.timetuple())
+        return _wrap_strftime(self, fmt)
 
     def __format__(self, fmt):
         if not isinstance(fmt, str):
@@ -577,7 +589,7 @@ date.resolution = _actual_datetime.timedelta(days=1)
 
 
 class datetime(date):
-    """datetime(year, month, day[, hour[, minute[, second[, microsecond[,tzinfo]]]]])
+    """datetime(year, month, day[, hour[, minute[, second[, microsecond[, tzinfo]]]]])
 
     The year, month and day arguments are required. tzinfo may be None, or an
     instance of a tzinfo subclass. The remaining arguments may be ints.
@@ -642,7 +654,7 @@ class datetime(date):
 
         converter = _time.gmtime if utc else _time.localtime
         y, m, d, hh, mm, ss, weekday, jday, dst = converter(t)
-        dt = cls.from_datetime_date(_actual_datetime.date(y, m, d))
+        dt = super().from_datetime_date(_actual_datetime.date(y, m, d))
         y, m, d = dt.year, dt.month, dt.day
         ss = min(ss, 59)  # clamp out leap seconds if the platform has them
         return cls(y, m, d, hh, mm, ss, us, tz)
@@ -705,9 +717,74 @@ class datetime(date):
             dst
         )
 
+    @classmethod
+    def from_datetime_datetime(cls, from_datetime):
+        """Convert datetime.date to nepali_datetime.datetime (A.D datetime to B.S).
+
+        Parameters
+        ----------
+        from_date: datetime.datetime
+            The AD datetime object to be converted.
+
+        Returns
+        -------
+        nepali_datetime.datetime
+            The converted nepali_datetime.datetime object.
+        """
+        from_datetime = from_datetime.astimezone(UTC0545())
+        return cls.combine(cls.from_datetime_date(from_datetime.date()), from_datetime.time())
+
+    def to_datetime_datetime(self):
+        """Convert nepali_datetime.datetime to datetime.datetime (B.S datetime to A.D).
+
+        Returns
+        -------
+        datetime.datetime
+            The converted datetime.datetime object.
+        """
+        return _actual_datetime.datetime.fromtimestamp(self.timestamp())
+
+    def _mktime(self):
+        """Return integer POSIX timestamp."""
+        max_fold_seconds = 24 * 3600
+        t = (self - _EPOCH_BS) // _actual_datetime.timedelta(0, 1)
+
+        def local(u):
+            y, m, d, hh, mm, ss = _time.localtime(u)[:6]
+            return (datetime(y, m, d, hh, mm, ss) - _EPOCH_BS) // _actual_datetime.timedelta(0, 1)
+
+        # Our goal is to solve t = local(u) for u.
+        a = local(t) - t
+        u1 = t - a
+        t1 = local(u1)
+        if t1 == t:
+            # We found one solution, but it may not be the one we need.
+            # Look for an earlier solution (if `fold` is 0), or a
+            # later one (if `fold` is 1).
+            u2 = u1 + (-max_fold_seconds, max_fold_seconds)[self.fold]
+            b = local(u2) - u2
+            if a == b:
+                return u1
+        else:
+            b = t1 - u1
+            assert a != b
+        u2 = t - b
+        t2 = local(u2)
+        if t2 == t:
+            return u2
+        if t1 == t:
+            return u1
+        # We have found both offsets a and b, but neither t - a nor t - b is
+        # a solution.  This means t is in the gap.
+        return (max, min)[self.fold](u1, u2)
+
     def timestamp(self):
         """Return POSIX timestamp as float"""
-        return NotImplemented
+        if self._tzinfo is None:
+            s = self._mktime()
+            return s + self.microsecond / 1e6
+        else:
+            return (self - _EPOCH_BS).total_seconds()
 
     def utctimetuple(self):
         """Return UTC time tuple compatible with time.gmtime()."""
@@ -752,7 +829,27 @@ class datetime(date):
         return datetime(year, month, day, hour, minute, second, microsecond, tzinfo)
 
     def astimezone(self, tz=None):
-        return NotImplemented
+        if tz is None:
+            tz = UTC0545()
+        elif not isinstance(tz, _actual_datetime.tzinfo):
+            raise TypeError("tz argument must be an instance of tzinfo")
+
+        mytz = self.tzinfo
+        if mytz is None:
+            mytz = self._local_timezone()
+            myoffset = mytz.utcoffset(self)
+        else:
+            myoffset = mytz.utcoffset(self)
+            if myoffset is None:
+                mytz = self.replace(tzinfo=None)._local_timezone()
+                myoffset = mytz.utcoffset(self)
+
+        if tz is mytz:
+            return self
+
+        utc = (self - myoffset).replace(tzinfo=tz)
+
+        return tz.fromutc(utc)
 
     def ctime(self):
         """Return ctime() style string."""
@@ -988,3 +1085,5 @@ class datetime(date):
 datetime.min = datetime(1975, 1, 1)
 datetime.max = datetime(2100, 12, 30, 23, 59, 59, 999999)
 datetime.resolution = _actual_datetime.timedelta(microseconds=1)
+
+_EPOCH_BS = datetime.from_datetime_datetime(_actual_datetime.datetime(1970, 1, 1, tzinfo=_actual_datetime.timezone.utc))
